﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

using Microsoft.Phone.Controls;
using C1.Phone.RichTextBox.Documents;
using C1.Phone.RichTextBox;

namespace RichTextBoxCopy
{
    public partial class MainPage : PhoneApplicationPage
    {
        // Constructor
        public MainPage()
        {
            InitializeComponent();
            LoadButtonIcons();

            _copyRichText = false;
        }      

        private void btnCopyPlainText_Click(object sender, RoutedEventArgs e)
        {
            // set clipboard text to plain text
            Clipboard.SetText(c1RichTextBox1.SelectedText);

            // keep focus on richtextbox
            c1RichTextBox1.Focus();
        }

        private void btnCopyRichText_Click(object sender, RoutedEventArgs e)
        {
            // set clipboard text to html text
            Clipboard.SetText(c1RichTextBox1.Selection.Html);

            // keep focus on richtextbox
            c1RichTextBox1.Focus();
        }

        private void c1RichTextBox1_SelectionChanged(object sender, EventArgs e)
        {
            PositionCopyButton();
        }

        private void PositionCopyButton()
        {
            if (c1RichTextBox1 == null)
                return;

            if (!string.IsNullOrEmpty(c1RichTextBox1.SelectedText))
            {
                // get bounds of selected text
                Rect rtStart = c1RichTextBox1.GetRectFromPosition(c1RichTextBox1.Selection.Start);
                Rect rtEnd = c1RichTextBox1.GetRectFromPosition(c1RichTextBox1.Selection.End);

                if (_copyRichText)
                {
                    // hide other button
                    btnCopyPlainText.Visibility = System.Windows.Visibility.Collapsed;

                    // center button above selected text
                    btnCopyRichText.Margin = new Thickness(rtStart.X + (Math.Abs(rtEnd.X - rtStart.X) / 2) - btnCopyRichText.Width / 2, rtStart.Y - btnCopyRichText.Height, 0, 0);
                    btnCopyRichText.Visibility = System.Windows.Visibility.Visible;
                }
                else
                {
                    // hide other button
                    btnCopyRichText.Visibility = System.Windows.Visibility.Collapsed;

                    // center button above selected text
                    btnCopyPlainText.Margin = new Thickness(rtStart.X + (Math.Abs(rtEnd.X - rtStart.X) / 2) - btnCopyPlainText.Width / 2, rtStart.Y - btnCopyPlainText.Height, 0, 0);
                    btnCopyPlainText.Visibility = System.Windows.Visibility.Visible;
                }
            }
            else
            {
                // hide button if no text to copy
                btnCopyPlainText.Visibility = System.Windows.Visibility.Collapsed;
                btnCopyRichText.Visibility = System.Windows.Visibility.Collapsed;
            }
        }

        private void radioPlainText_Checked(object sender, RoutedEventArgs e)
        {
            _copyRichText = false;
            PositionCopyButton();
        }

        private void radioRichText_Checked(object sender, RoutedEventArgs e)
        {
            _copyRichText = true;
            PositionCopyButton();
        }

        bool _copyRichText;
        bool _updating; 
        Storyboard _storyBoard;
        C1TextRange _range;

        private void c1RichTextBox1_TextChanged(object sender, C1TextChangedEventArgs e)
        {
            // handle pasting html
            if (e.Range.Text.Contains("<html>") && _copyRichText)
            {
                // start timer whenever document receives html text from clipboard
                // we can't directly change text within this event so we do it in storyboard_Completed event
                if (!_updating)
                {
                    _range = e.Range;
                    if (_storyBoard == null)
                    {
                        _storyBoard = new Storyboard();
                        _storyBoard.Completed += new EventHandler(storyboard_Completed);
                        _storyBoard.Duration = new Duration(TimeSpan.FromMilliseconds(1));
                    }
                    _storyBoard.Stop();
                    _storyBoard.Seek(TimeSpan.Zero);
                    _storyBoard.Begin();
                }               
            }
        }

        void storyboard_Completed(object sender, EventArgs e)
        {
            _updating = true;
            UpdatePastedText(_range);
            _updating = false;
        }

        private void UpdatePastedText(C1TextRange range)
        {
            // remove unnecessary paragraph tag
            range.Text = range.Text.Remove(range.Text.LastIndexOf("</p>"), 4);
            int p = range.Text.LastIndexOf("<p");
            string pTag = range.Text.Substring(p, 20); // assumes > tag is within 20 characters of <p
            range.Text = range.Text.Remove(p, pTag.IndexOf(">") + 1);

            // sets rich html text from clipboard into the text range
            range.Html = range.Text;

            // put caret at the end of pasted selection
            c1RichTextBox1.Select(range.End.TextOffset, 0);
        }

        private void LoadButtonIcons()
        {
            // change button icons if using light theme
            if ((Color)Application.Current.Resources["PhoneBackgroundColor"] == Colors.White)
            {
                imgPlainText.Source = new BitmapImage(new Uri("Resources/Copy-Light.png", UriKind.Relative));
                imgRichText.Source = new BitmapImage(new Uri("Resources/CopyStyles-Light.png", UriKind.Relative));
            }
        }
    }
}